<?php defined('SYSPATH') or die('No direct script access.');

class FileParser
{
    /**
     * Parse students' info & synchronize it with database.
     * @param $filename string File with source data
     * @param $facultyID int
     * @return array errors
     */
    public static function uploadStudents($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");

        $i = 0;
        $errors = [];

        while ($line = fgets($file)) {
            $line = explode(';', self::toUTF8($line));

            // ФИО, курс, степень подготовки,
            list($name, $gradeNum, $groupNum, $degree, $spec) = UTF8::clear($line);

            // Проверим, что входная строка не пуста.
            $name = trim($name);
            if (empty($name))
                continue;

            // Фамилия, имя, отчество
            list($lastName, $firstName, $secondName) = Text::parseFullName($name);

            $degree = self::getDegreeType($degree);

            $attempt = Model_Account::createStudentEx(
                $lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $spec, $facultyID
            );

            if ($attempt == -1) {
                $errors[] = ['Row' => $i, 'Info' => implode(';', $line)];
            }

            $i++;
        }

        return $errors;
    }

    private static function getDegreeType($degree) {
        switch ($degree) {
            case 'Бакалавр':
                return 'bachelor';
            case 'Магистр':
                return 'master';
            case 'Специалист':
                return 'specialist';
            default:
                return $degree;
        }
    }

    /**
     * Parse subjects' info & synchronize it with database.
     * @param $filename  string File with source data
     * @param $facultyID int
     * @return array
     */
    public static function uploadSubjects($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");
        $count = $exists = 0;
        $errors = [];

        while ($line = fgets($file)) {

            $line = explode(';', self::toUTF8($line));

            list($name, $abbr) = UTF8::clear($line);

            // Проверим, что входная строка не пуста.
            $name = trim($name);
            if (empty($name))
                continue;

            $attempt = Model_Subject::create($name, $abbr, $facultyID);

            if ($attempt < 0) {
                $errors[] = ['Row' => $count, 'Info' => implode(';', $line)];
            } elseif ($attempt == 1) {
                $exists++;
            }

            $count++;
        }

        return [
            'Errors'             => $errors,
            'ErrorsCount'        => count($errors),
            'RecordsCount'       => $count,
            'RecordsExistsCount' => $exists,
        ];
    }

    public static function toUTF8($source_str) {
        return iconv(mb_detect_encoding($source_str, mb_detect_order(), true), "UTF-8", $source_str);
    }
}
